﻿using System;
using System.Runtime.InteropServices;

using Windows.System;

namespace TestProject
{
    /// <summary>
    /// 윈도우즈 시스템 디스패처 큐 헬퍼
    /// </summary>
    public class WindowsSystemDispatcherQueueHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Structure
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 디스패처 큐 옵션 - DispatcherQueueOptions

        /// <summary>
        /// 디스패처 큐 옵션
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        private struct DispatcherQueueOptions
        {
            //////////////////////////////////////////////////////////////////////////////////////////////////// Field
            ////////////////////////////////////////////////////////////////////////////////////////// Internal

            #region Field

            /// <summary>
            /// 크기
            /// </summary>
            internal int Size;

            /// <summary>
            /// 쓰레드 타입
            /// </summary>
            internal int ThreadType;

            /// <summary>
            /// 아파트먼트 타입
            /// </summary>
            internal int ApartmentType;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Import
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 디스패처 큐 컨트롤러 생성하기 - CreateDispatcherQueueController(options, instance)

        /// <summary>
        /// 디스패처 큐 컨트롤러 생성하기
        /// </summary>
        /// <param name="options">디스패처 큐 옵션</param>
        /// <param name="instance">인스턴스</param>
        /// <returns>처리 결과</returns>
        [DllImport("CoreMessaging")]
        private static unsafe extern int CreateDispatcherQueueController(DispatcherQueueOptions options, IntPtr* instance);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 디스패처 큐 컨트롤러 핸들
        /// </summary>
        private IntPtr dispatcherQueueControllerHandle = IntPtr.Zero;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 윈도우즈 시스템 디스패처 큐 컨트롤러 보장하기 - EnsureWindowsSystemDispatcherQueueController()

        /// <summary>
        /// 윈도우즈 시스템 디스패처 큐 컨트롤러 보장하기
        /// </summary>
        public void EnsureWindowsSystemDispatcherQueueController()
        {
            if(DispatcherQueue.GetForCurrentThread() != null)
            {
                return;
            }

            if(this.dispatcherQueueControllerHandle == IntPtr.Zero)
            {
                DispatcherQueueOptions options;

                options.Size          = Marshal.SizeOf(typeof(DispatcherQueueOptions));
                options.ThreadType    = 2; // DQTYPE_THREAD_CURRENT
                options.ApartmentType = 2; // DQTAT_COM_STA

                unsafe
                {
                    IntPtr dispatcherQueueController;

                    CreateDispatcherQueueController(options, &dispatcherQueueController);

                    this.dispatcherQueueControllerHandle = dispatcherQueueController;
                }
            }
        }

        #endregion
    }
}